﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;

namespace VeeRifter
{
    public class AutoLabia : MVRScript
    {
		private static FreeControllerV3 chest;
		private static FreeControllerV3 lKnee;
		private static FreeControllerV3 rKnee;
		protected JSONStorableFloat labMorphMin;
		protected JSONStorableFloat labMorphMax;		
		protected JSONStorableFloat MajUpMin;
		protected JSONStorableFloat MajUpMax;	
		protected JSONStorableFloat MajMidMin;
		protected JSONStorableFloat MajMidMax;
		protected JSONStorableFloat MajLowMin;
		protected JSONStorableFloat MajLowMax;
		protected JSONStorableFloat MinUpMin;
		protected JSONStorableFloat MinUpMax;	
		protected JSONStorableFloat MinMidMin;
		protected JSONStorableFloat MinMidMax;
		protected JSONStorableFloat MinLowMin;
		protected JSONStorableFloat MinLowMax;
		private Color paleMagenta = new Color(0.98f,0.75f,0.83f,1.0f);
		private Color darkMagenta = new Color(0.43f,0.0f,0.49f,1.0f);		
		private Color paleBlue = new Color(0.75f,0.93f,1.0f,1.0f);
		private Color darkBlue = new Color(0.0f,0.0f,0.7f,1.0f);
		private Color paleYellow = new Color(0.98f,1.0f,0.54f,1.0f);
		private Color orange = new Color(1.0f,0.5f,0.0f,1.0f);		
		private const string _saveExt = "preset";
		private const string _presetSavePath = "Saves\\PluginData\\AutoLabia";
		private UIDynamic slider;
		
		private List<string> morphNames = new List<string>
        {
			"Genitals-extreme expansion",
            "Labia majora-spread-LUp",
			"Labia majora-spread-RUp",
            "Labia majora-spread-LMid",
			"Labia majora-spread-RMid",
            "Labia majora-spread-LLow",
			"Labia majora-spread-RLow",
	        "Labia minora-spread-LUp",
			"Labia minora-spread-RUp",
            "Labia minora-spread-LMid",
			"Labia minora-spread-RMid",
            "Labia minora-spread-LLow",
			"Labia minora-spread-RLow",	
        };
		
		private	List<JSONStorableFloat> minVals = new List<JSONStorableFloat>{};		
		private	List<JSONStorableFloat> maxVals = new List<JSONStorableFloat>{};
		private DAZMorph[] morphs = new DAZMorph[13];		
			
        public override void Init()
        {
            try
            {
				if (containingAtom.type != "Person")
				{
					SuperController.LogError($"Plugin for use with 'Person' atom, not '{containingAtom.type}'");
					return;
				}
	
				pluginLabelJSON.val = "AutoLabia v4.0 - VeeRifter";			
                JSONStorable js = containingAtom.GetStorableByID("geometry");
                if (js != null)
                {
                    DAZCharacterSelector dcs = js as DAZCharacterSelector;
                    GenerateDAZMorphsControlUI morphUI = dcs.morphsControlUI;
                    if (morphUI != null)
                    {
						for (int index = 0; index < morphs.Length; index++) {
							morphs[index] = null;
							morphs[index] = morphUI.GetMorphByDisplayName(morphNames[index]);							
							if (morphs[index] == null)
							{
								SuperController.LogError("Couldn't find morph '" + morphNames[index] + "'");
								return;
							}							
						}
                    }
                }
			
				chest = containingAtom.GetStorableByID("chestControl") as FreeControllerV3;
				lKnee = containingAtom.GetStorableByID("lKneeControl") as FreeControllerV3;
				rKnee = containingAtom.GetStorableByID("rKneeControl") as FreeControllerV3;													

				var headingText1 = new JSONStorableString("headingText1","<i><b>Legs Closed Settings</b></i>");				
				UIDynamicTextField dtfHeading1 = CreateCustomTextField(headingText1, 45, false);
				dtfHeading1.UItext.fontSize = 36;
				dtfHeading1.backgroundColor = Color.white;
				dtfHeading1.textColor = Color.black;
				
				var headingText2 = new JSONStorableString("headingText2","<i><b>Legs Open Settings</b></i>");				
				UIDynamicTextField dtfHeading2 = CreateCustomTextField(headingText2, 45, true);
				dtfHeading2.UItext.fontSize = 36;
				dtfHeading2.backgroundColor = Color.white;
				dtfHeading2.textColor = Color.black;

				labMorphMin = new JSONStorableFloat("Labia Morph Min", 0f, -1f, 2f, false, true);
				labMorphMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(labMorphMin);
				slider = CreateSlider(labMorphMin, false);			
				setSliderColors(paleYellow, orange);	
				minVals.Add(labMorphMin);
				
				labMorphMax = new JSONStorableFloat("Labia Morph Max", 0f, -1f, 2f, false, true);
				labMorphMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(labMorphMax);
				slider = CreateSlider(labMorphMax, true);			
				setSliderColors(paleYellow, orange);				
				maxVals.Add(labMorphMax);

				
				var headingText3 = new JSONStorableString("headingText3","<i><b>Labia Majora</b></i>");				
				UIDynamicTextField dtfHeading3 = CreateCustomTextField(headingText3, 45, false);
				dtfHeading3.UItext.fontSize = 36;
				dtfHeading3.backgroundColor = paleBlue;
				dtfHeading3.textColor = darkBlue;
				
				var headingText4 = new JSONStorableString("headingText4","<i><b>Labia Majora</b></i>");				
				UIDynamicTextField dtfHeading4 = CreateCustomTextField(headingText4, 45, true);
				dtfHeading4.UItext.fontSize = 36;
				dtfHeading4.backgroundColor = paleBlue;
				dtfHeading4.textColor = darkBlue;
				
				MajUpMin = new JSONStorableFloat("Majora Upper Min", 0f, -1f, 2f, false, true);
				MajUpMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajUpMin);
				slider = CreateSlider(MajUpMin, false);			
				setSliderColors(paleBlue, darkBlue);	
				minVals.Add(MajUpMin);
				minVals.Add(MajUpMin);
				
				MajUpMax = new JSONStorableFloat("Majora Upper Max", 0f, -1f, 2f, false, true);
				MajUpMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajUpMax);
				slider = CreateSlider(MajUpMax, true);			
				setSliderColors(paleBlue, darkBlue);				
				maxVals.Add(MajUpMax);
				maxVals.Add(MajUpMax);
				
				MajMidMin = new JSONStorableFloat("Majora Middle Min", 0f, -1f, 2f, false, true);
				MajMidMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajMidMin);
				slider = CreateSlider(MajMidMin, false);				
				setSliderColors(paleBlue, darkBlue);		
				minVals.Add(MajMidMin);
				minVals.Add(MajMidMin);
			
				MajMidMax = new JSONStorableFloat("Majora Middle Max", 0f, -1f, 2f, false, true);
				MajMidMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajMidMax);
				slider = CreateSlider(MajMidMax, true);			
				setSliderColors(paleBlue, darkBlue);			
				maxVals.Add(MajMidMax);
				maxVals.Add(MajMidMax);
				
				MajLowMin = new JSONStorableFloat("Majora Lower Min", 0f, -1f, 2f, false, true);
				MajLowMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajLowMin);
				slider = CreateSlider(MajLowMin, false);			
				setSliderColors(paleBlue, darkBlue);				
				minVals.Add(MajLowMin);
				minVals.Add(MajLowMin);
				
				MajLowMax = new JSONStorableFloat("Majora Lower Max", 0f, -1f, 2f, false, true);
				MajLowMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MajLowMax);
				slider = CreateSlider(MajLowMax, true);				
				setSliderColors(paleBlue, darkBlue);				
				maxVals.Add(MajLowMax);
				maxVals.Add(MajLowMax);					
				
				var headingText5 = new JSONStorableString("headingText5","<i><b>Labia Minora</b></i>");				
				UIDynamicTextField dtfHeading5 = CreateCustomTextField(headingText5, 45, false);
				dtfHeading5.UItext.fontSize = 36;
				dtfHeading5.backgroundColor = paleMagenta;							
				dtfHeading5.textColor = darkMagenta;
				
				var headingText6 = new JSONStorableString("headingText6","<i><b>Labia Minora</b></i>");				
				UIDynamicTextField dtfHeading6 = CreateCustomTextField(headingText6, 45, true);
				dtfHeading6.UItext.fontSize = 36;
				dtfHeading6.backgroundColor = paleMagenta;				
				dtfHeading6.textColor = darkMagenta;
				
				MinUpMin = new JSONStorableFloat("Minora Upper Min", 0f, -1f, 2f, false, true);
				MinUpMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinUpMin);
				slider = CreateSlider(MinUpMin, false);			
				setSliderColors(paleMagenta, darkMagenta);		
				minVals.Add(MinUpMin);
				minVals.Add(MinUpMin);
				
				MinUpMax = new JSONStorableFloat("Minora Upper Max", 0f, -1f, 2f, false, true);
				MinUpMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinUpMax);
				slider = CreateSlider(MinUpMax, true);			
				setSliderColors(paleMagenta, darkMagenta);		
				maxVals.Add(MinUpMax);
				maxVals.Add(MinUpMax);
				
				MinMidMin = new JSONStorableFloat("Minora Middle Min", 0f, -1f, 2f, false, true);
				MinMidMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinMidMin);
				slider = CreateSlider(MinMidMin, false);			
				setSliderColors(paleMagenta, darkMagenta);			
				minVals.Add(MinMidMin);
				minVals.Add(MinMidMin);
				
				MinMidMax = new JSONStorableFloat("Minora Middle Max", 0f, -1f, 2f, false, true);
				MinMidMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinMidMax);
				slider = CreateSlider(MinMidMax, true);
				setSliderColors(paleMagenta, darkMagenta);			
				maxVals.Add(MinMidMax);
				maxVals.Add(MinMidMax);

				MinLowMin = new JSONStorableFloat("Minora Lower Min", 0f, -1f, 2f, false, true);
				MinLowMin.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinLowMin);
				slider = CreateSlider(MinLowMin, false);				
				setSliderColors(paleMagenta, darkMagenta);				
				minVals.Add(MinLowMin);
				minVals.Add(MinLowMin);
				
				MinLowMax = new JSONStorableFloat("Minora Lower Max", 0f, -1f, 2f, false, true);
				MinLowMax.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(MinLowMax);
				slider = CreateSlider(MinLowMax, true);			
				setSliderColors(paleMagenta, darkMagenta);			
				maxVals.Add(MinLowMax);
				maxVals.Add(MinLowMax);
				
				var savePresetUI = CreateButton("<b><i>Save Preset</i></b>", false);
				savePresetUI.buttonText.color = paleBlue;
				savePresetUI.buttonColor = darkBlue;			
				savePresetUI.button.onClick.AddListener(openSavePresetUI);
				
				var loadPresetUI = CreateButton("<b><i>Load Preset</i></b>", true);
				loadPresetUI.buttonText.color = paleBlue;
				loadPresetUI.buttonColor = darkBlue;				
				loadPresetUI.button.onClick.AddListener(openLoadPresetUI);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught in Init(): " + e);
            }
        }

        public void Update()
        {	
			Vector3 chestPos = chest.followWhenOff.position;			
			Vector3 lKneePos = lKnee.followWhenOff.position;
			Vector3 rKneePos = rKnee.followWhenOff.position;
			Vector3 direction = lKneePos - rKneePos;
			float rKneeTolKnee = direction.magnitude;
			direction = lKneePos - chestPos;
			float lKneeToChest = direction.magnitude;
			direction = rKneePos - chestPos;
			float rKneeToChest = direction.magnitude;
			float spread = rKneeTolKnee  - (Mathf.Abs(lKneeToChest - rKneeToChest) * 2);
			
			morphs[0].morphValue = Remap(spread, 0, 1, minVals[0].val, maxVals[0].val);//Legacy morph from earlier versions
			for (int index = 1; index < morphs.Length; ++index)
			{			
				morphs[index].morphValue = Remap(spread, 0, 1, minVals[index].val, maxVals[index].val);			
				index++;
				morphs[index].morphValue = Remap(spread, 0, 1, minVals[index].val, maxVals[index].val);	
			}		
        }
		
        private float Remap(float inVal, float min1, float max1, float min2, float max2)
        {
            var ratio = (max2 - min2) / (max1 - min1);
            var c = min2 - ratio * min1;			
            return ratio * inVal + c;
        }

		private void openSavePresetUI()
        {
            FileManagerSecure.CreateDirectory(_presetSavePath);
            var fileBrowserUI = SuperController.singleton.fileBrowserUI;
            fileBrowserUI.SetTitle("Save AutoLabia Preset");
            fileBrowserUI.fileRemovePrefix = null;
            fileBrowserUI.hideExtension = false;
            fileBrowserUI.keepOpen = false;
            fileBrowserUI.fileFormat = _saveExt;
            fileBrowserUI.defaultPath = _presetSavePath;
            fileBrowserUI.showDirs = true;
            fileBrowserUI.shortCuts = null;
            fileBrowserUI.browseVarFilesAsDirectories = false;
            fileBrowserUI.SetTextEntry(true);
            fileBrowserUI.Show(savePreset);
            fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + "." + _saveExt;
            fileBrowserUI.ActivateFileNameField();
        }
		
		private void savePreset(string path)
		{
			SuperController.singleton.fileBrowserUI.fileFormat = null;
			if (string.IsNullOrEmpty(path)) return;
			if (!path.ToLower().EndsWith($".{_saveExt}")) path += $".{_saveExt}";

			if (!path.ToLower().EndsWith($".{_saveExt}"))
				path += $".{_saveExt}";

			JSONClass jscPreset = new JSONClass();				
			for (int index = 0; index < minVals.Count; ++index)
			{
				jscPreset["minVals_" + index.ToString()] = minVals[index].val.ToString();				
			}					
			for (int index = 0; index < maxVals.Count; ++index)
			{
				jscPreset["maxVals_" + index.ToString()] = maxVals[index].val.ToString();				
			}		
			SaveJSON(jscPreset, path);
		}
		
		private void openLoadPresetUI()		
        {
            FileManagerSecure.CreateDirectory(_presetSavePath);
            var presets = FileManagerSecure.GetShortCutsForDirectory(_presetSavePath);
            SuperController.singleton.GetMediaPathDialog(loadPreset, _saveExt, _presetSavePath, false, true, false, null, false, presets);
        }		
	
		private void loadPreset(string path)
		{
			if (string.IsNullOrEmpty(path))
				return;
			JSONClass jscPreset = LoadJSON(path).AsObject;


			if (jscPreset.Count == minVals.Count + maxVals.Count)
			{
				for (int index = 0; index < minVals.Count; ++index)
				{	
					minVals[index].val = Convert.ToSingle(jscPreset[index]);				
				}
				for (int index = 0; index < maxVals.Count; ++index)
				{
					maxVals[index].val = Convert.ToSingle(jscPreset[index + minVals.Count]);			
				}
			}
			else
			{
				SuperController.LogError("Invalid number of entries in preset '" + path + "'");
			}
		}


        public UIDynamicTextField CreateCustomTextField(JSONStorableString jss, float Height = 40f, bool rightSide = false) 
	    {	
            var container = new GameObject();
            container.transform.SetParent(transform, false);
            {
                var rect = container.AddComponent<RectTransform>();
                rect.pivot = new Vector2(0, 1);
                var layout = container.AddComponent<LayoutElement>();
                layout.preferredHeight = Height;				
                layout.flexibleWidth = 1f;
            }
            var textfield = Instantiate(this.manager.configurableTextFieldPrefab).GetComponent<UIDynamicTextField>();
            textfield.gameObject.transform.SetParent(container.transform, false);
            {
                jss.dynamicText = textfield;							
                var rect = textfield.GetComponent<RectTransform>();
                rect.anchorMin = new Vector2(0f, 1f);			
                rect.anchorMax = new Vector2(1f, 1f);
                rect.pivot = new Vector2(0f, 1f);
                rect.anchoredPosition = new Vector2(0f, 0f);
                rect.sizeDelta = new Vector2(0f, 50f);
            }		
			if (rightSide)
				rightUIElements.Add(container.transform);
			else
				leftUIElements.Add(container.transform);
			
			textfield.UItext.supportRichText = true;
			textfield.backgroundColor = Color.white;
			textfield.UItext.color = Color.black;
			textfield.UItext.alignment = TextAnchor.MiddleCenter;
			textfield.UItext.fontSize = 30;			
			textfield.height = Height;			
            return textfield;
        }

        private void setSliderColors(Color color1, Color color2)
        {
            if (slider != null)
            {
                Image[] sliderImages = slider.GetComponentsInChildren<Image>();
                foreach (var image in sliderImages)
                {
					switch (image.name)
					{
						case "Fill":
							image.color = color1;
							break;
						case "Handle":
							image.color = color2;
							break;
						default:
							break;
					}										
                }					
            }
        }	
    }
}
